@@ -293,14 +293,8 @@
 	NJBQueueItem *copyTrack = [[NJBQueueItem alloc] initWithTarget:theNJB withSelector:@selector(uploadTrack:) withObject:track];
 	[copyTrack setStatus:STATUS_UPLOADING_TRACK];
 	
-	NJBQueueItem *addToTrackList = [[NJBQueueItem alloc] initWithTarget:fullItemArray withSelector:@selector(addObject:) withObject:track withRunInMainThread:YES];
-	[addToTrackList setDisplayStatus:NO];
-	
-	// call searchNJBTable to pretend the search list changed to update
-	// saves data and does a reload
-	NJBQueueItem *updateTrackList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(searchNJBTable)];
-	[updateTrackList setRunInMainThread:YES];
-	[updateTrackList setDisplayStatus:NO];
+	NJBQueueItem *addToList = [[NJBQueueItem alloc] initWithTarget:self withSelector:@selector(addToFullItemArray:) withObject:track withRunInMainThread:YES];
+	[addToList setDisplayStatus:NO];
 	
 	// tell PlaylistsTab
 	NJBQueueItem *updatePlaylistsTab = [[NJBQueueItem alloc] initWithTarget:playlistsTab withSelector:@selector(updateTracks)];
@@ -308,11 +302,10 @@
 	[updatePlaylistsTab setDisplayStatus:NO];
 	
 	NSString *description = [NSString stringWithFormat:@"Uploading track '%@'", [track title]];
-	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToTrackList, updateTrackList, updatePlaylistsTab, nil] withDescription:description];
+	[self addToQueue:copyTrack withSubItems:[NSArray arrayWithObjects:addToList, updatePlaylistsTab, nil] withDescription:description];
 	
 	[copyTrack release];
-	[addToTrackList release];
-	[updateTrackList release];
+	[addToList release];
 	[updatePlaylistsTab release];
 }
 
