@@ -20,14 +20,6 @@
 	IBOutlet NSButton *changeMusicTabDir;
 	IBOutlet NSButton *changeDataTabDir;
 	IBOutlet NSButton *writeTagAfterCopy;
-	IBOutlet NSTextField *duplicatesLengthTol;
-	IBOutlet NSTextField *duplicatesFilesizeTol;
-	IBOutlet NSTextField *duplicatesTitleTol;
-	IBOutlet NSTextField *duplicatesArtistTol;
-	IBOutlet NSStepper *duplicatesLengthTolStepper;
-	IBOutlet NSStepper *duplicatesFilesizeTolStepper;
-	IBOutlet NSStepper *duplicatesTitleTolStepper;
-	IBOutlet NSStepper *duplicatesArtistTolStepper;
 	Preferences *preferences;
 	NSTabView *tabView;
 	BOOL isDirty;
@@ -48,5 +40,4 @@
 - (IBAction)musicTabDirLastUsed:(id)sender;
 - (IBAction)dataTabDirLastUsed:(id)sender;
 - (IBAction)writeTagAfterCopy:(id)sender;
-
 @end
