@@ -1,9 +1,5 @@
 #include <libnjb.h>
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#else
-#include <unistd.h>
-#endif
 
 void usage (void);
 
