@@ -321,11 +321,11 @@ static void add_bin_unistr(unsigned char *data, u_int32_t *datap, u_int32_t tagt
 unsigned char *datafile_pack3 (njb_t *njb, njb_datafile_t *df, u_int32_t *size)
 {
   __dsub= "datafile_pack3";
+  unsigned char *ptag;
   unsigned char *filename = NULL;
   unsigned char *foldername = NULL;
-  unsigned char ptag[1024];
-  unsigned char *retag;
   u_int32_t p = 0;
+  u_int16_t len;
   
   __enter;
   
@@ -350,23 +350,36 @@ unsigned char *datafile_pack3 (njb_t *njb, njb_datafile_t *df, u_int32_t *size)
     return NULL;
   }
  
+  len = (u_int16_t) ucs2strlen(filename)*2 + ucs2strlen(foldername)*2;
+  if ( njb->device_type == NJB_DEVICE_NJB3 
+       || njb->device_type == NJB_DEVICE_NJBZEN ) {
+    *size = len + 34; /* Misc metadata magic size 34 */
+  } else {
+    *size = len + 36; /* Misc metadata magic size 36 */
+  }
+  
+  ptag = (unsigned char *) malloc(*size);
+  if ( ptag == NULL ) {
+    free(filename);
+    NJB_ERROR(njb, EO_NOMEM);
+    __leave;
+    return NULL;
+  }
+  
   /* Add filename tag */
   add_bin_unistr(ptag, &p, NJB3_FNAME_FRAME_ID, filename);
   free(filename);
   /* Add folder tag */
   add_bin_unistr(ptag, &p, NJB3_DIR_FRAME_ID, foldername);
   free(foldername);
-
   /* Add filesize in 32 bits */
   from_16bit_to_njb3_bytes(0x0006, &ptag[p]);
   p += 2;
   from_16bit_to_njb3_bytes(NJB3_FILESIZE_FRAME_ID, &ptag[p]);
   p += 2;
-  /*
-   * NJB3 doesn't seem to support
+  /* NJB3 doesn't seem to support
    * filesizes greater than 32 bits (?) 
-   * so df->msdw is thrown away! 
-   */
+   * so df->msdw is thrown away! */
   from_32bit_to_njb3_bytes((u_int32_t) df->filesize, &ptag[p]);
   p += 4;
   /* Add timestamp. */
@@ -380,35 +393,30 @@ unsigned char *datafile_pack3 (njb_t *njb, njb_datafile_t *df, u_int32_t *size)
        || njb->device_type == NJB_DEVICE_NJBZEN 
        ) {
     /* This is the locked status used in older firmware. The file is not locked. */
-    from_16bit_to_njb3_bytes(0x0004U, &ptag[p]);
+    from_16bit_to_njb3_bytes(0x0004, &ptag[p]);
     p += 2;
     from_16bit_to_njb3_bytes(NJB3_LOCKED_FRAME_ID, &ptag[p]);
     p += 2;
-    from_16bit_to_njb3_bytes(0x0000U, &ptag[p]); /* = not locked */
+    from_16bit_to_njb3_bytes(0x0000, &ptag[p]); /* = not locked */
+    p += 2;
+    from_16bit_to_njb3_bytes(0x0000, &ptag[p]); /* Terminator */
     p += 2;
   } else {
     /* End with flags instead - used in USB 2.0 devices
      * Probably locking files was deemed unnecessary,
      * so that was removed. Instead this strange thing was introduced. */
-    from_16bit_to_njb3_bytes(0x0006U, &ptag[p]);
+    from_16bit_to_njb3_bytes(0x0006, &ptag[p]);
     p += 2;
     from_16bit_to_njb3_bytes(NJB3_FILEFLAGS_FRAME_ID, &ptag[p]); /* Could be readable/writeable etc... */
     p += 2;
     from_32bit_to_njb3_bytes(0x20000000U, &ptag[p]); /* Frame contents */
     p += 4;
+    from_16bit_to_njb3_bytes(0x0000, &ptag[p]); /* Terminator */
+    p += 2;
   }
-
-  *size = p;
-  retag = (unsigned char *) malloc(*size);
-  if ( retag == NULL ) {
-    NJB_ERROR(njb, EO_NOMEM);
-    __leave;
-    return NULL;
-  }
-  memcpy(retag, ptag, *size);
   
   __leave;
-  return retag;
+  return ptag;
 }
 
 /**
@@ -426,10 +434,10 @@ unsigned char *datafile_pack3 (njb_t *njb, njb_datafile_t *df, u_int32_t *size)
 unsigned char *new_folder_pack3 (njb_t *njb, const char *name, u_int32_t *size)
 {
   __dsub= "new_folder_pack3";
-  unsigned char ptag[1024];
-  unsigned char *retag;
+  unsigned char *ptag;
   unsigned char *dirname = NULL;
   u_int32_t p = 0;
+  u_int16_t len;
   
   __enter;
   
@@ -441,6 +449,22 @@ unsigned char *new_folder_pack3 (njb_t *njb, const char *name, u_int32_t *size)
     return NULL;
   }
   
+  len = (u_int16_t) ucs2strlen(dirname)*2;
+  if ( njb->device_type == NJB_DEVICE_NJB3 
+       || njb->device_type == NJB_DEVICE_NJBZEN ) {
+    *size = len + 42; /* Misc metadata magic size 42 */
+  } else {
+    *size = len + 44; /* Misc metadata magic size 44 */
+  }
+  
+  ptag = (unsigned char *) malloc(*size);
+  if ( ptag == NULL ) {
+    free(dirname);
+    NJB_ERROR(njb, EO_NOMEM);
+    __leave;
+    return NULL;
+  }
+  
   /* Add "." period filename tag */
   from_16bit_to_njb3_bytes(0x0006, &ptag[p]);
   p += 2;
@@ -478,6 +502,8 @@ unsigned char *new_folder_pack3 (njb_t *njb, const char *name, u_int32_t *size)
     p += 2;
     from_16bit_to_njb3_bytes(0x0000, &ptag[p]); /* = not locked */
     p += 2;
+    from_16bit_to_njb3_bytes(0x0000, &ptag[p]); /* Terminator */
+    p += 2;
   } else {
     /* End with flags instead - used in USB 2.0 devices
      * Probably locking files was deemed unnecessary,
@@ -488,24 +514,14 @@ unsigned char *new_folder_pack3 (njb_t *njb, const char *name, u_int32_t *size)
     p += 2;
     from_32bit_to_njb3_bytes(0x80000000U, &ptag[p]); /* Frame contents */
     p += 4;
+    from_16bit_to_njb3_bytes(0x0000, &ptag[p]); /* Terminator */
+    p += 2;
   }
-  /* Three 0x0000 after the stuff, why, I don't know. */
   from_16bit_to_njb3_bytes(0x0000, &ptag[p]);
   p += 2;
   from_16bit_to_njb3_bytes(0x0000, &ptag[p]);
   p += 2;
-  from_16bit_to_njb3_bytes(0x0000, &ptag[p]);
-  p += 2;
-
-  *size = p;
-  retag = (unsigned char *) malloc(*size);
-  if ( retag == NULL ) {
-    NJB_ERROR(njb, EO_NOMEM);
-    __leave;
-    return NULL;
-  }
-  memcpy(retag, ptag, *size);
   
   __leave;
-  return retag;
+  return ptag;
 }
